import .syntax

namespace desugar_wf

@[simp] def psum_simp {t u w z : Type} : psum (Σ' (a : t), u) (Σ' (a : w), z) → psum t w
| (psum.inr s) := psum.inr s.fst
| (psum.inl s) := psum.inl s.fst

instance hwf_instance :  has_well_founded
    (psum (Σ' (a : sugar.expr), string) (Σ' (a : list sugar.expr), imp.stmt × list imp.expr × string)) :=
begin
  constructor,
  tactic.rotate_left 1,
  refine (measure _),
  intros,
  have x := (psum_simp a),
  exact (sizeof x),
  apply measure_wf,
end

open well_founded_tactics
open tactic

@[simp] def psum_inl_inl (e e' : sugar.expr) : sizeof e < sizeof e' → psum.sizeof (@psum.inl _ (list sugar.expr) e) < psum.sizeof (@psum.inl _ (list sugar.expr) e') :=
begin
  intros,
  simp [psum.sizeof],
  cancel_nat_add_lt,
  assumption
end

@[simp] def psum_inr_inl (e : list sugar.expr) (e' : sugar.expr) : sizeof e < sizeof e' → psum.sizeof (@psum.inr sugar.expr _ e) < psum.sizeof (@psum.inl _ (list sugar.expr) e') :=
begin
  intros,
  simp [psum.sizeof],
  cancel_nat_add_lt,
  assumption
end

@[simp] def psum_inl_inr (e : list sugar.expr) (e' : sugar.expr) : sizeof e' < sizeof e →
psum.sizeof (@psum.inl _ (list sugar.expr) e') < psum.sizeof (@psum.inr sugar.expr _ e) :=
begin
  intros,
  simp [psum.sizeof],
  cancel_nat_add_lt,
  assumption
end

@[simp] def psum_inr_inr (e e' : list sugar.expr) : sizeof e < sizeof e' → psum.sizeof (@psum.inr (sugar.expr) _ e) < psum.sizeof (@psum.inr sugar.expr _ e') :=
begin
  intros,
  simp [psum.sizeof],
  cancel_nat_add_lt,
  assumption
end

meta def my_dec_tac : tactic unit :=
do clear_internals,
   unfold_wf_rel,
   unfold_sizeof,
   cancel_nat_add_lt,
   -- need to apply these rules
   `[apply desugar_wf.psum_inl_inl] <|>
   `[apply desugar_wf.psum_inr_inl] <|>
   `[apply desugar_wf.psum_inl_inr] <|>
   `[apply desugar_wf.psum_inr_inr],
   default_dec_tac

meta def tactics : well_founded_tactics :=
{ rel_tac := fun f eqns, mk_const `desugar_wf.hwf_instance >>= apply , dec_tac := my_dec_tac }

end desugar_wf
