import ..to_python
import ..syntax
import .has_to_string

namespace imp

def stmt.to_python' : stmt → list string
| (stmt.nop) := "" :: []
| (stmt.set x e) := [to_string x ++ " = " ++ to_string e]
| (stmt.alloc x e1 e2) := [to_string x ++ " = " ++ to_string e1 ++ " * [" ++ to_string e2 ++ "]"]
| (stmt.write x e1 e2) := [to_string x ++ "[" ++ to_string e1 ++ "] = " ++ to_string e2]
| (stmt.call x f es) :=
  [x ++ " = " ++ to_string f ++ (parens $ concat ", " $ es.map expr.to_string)]
| (stmt.seq s1 s2) := stmt.to_python' s1 ++ stmt.to_python' s2
| (stmt.ifelse e p1 stmt.nop) :=
  ["if " ++ to_string e ++ ":"] ++ indent (stmt.to_python' p1)
| (stmt.ifelse e p1 p2) :=
  ["if " ++ to_string e ++ ":"] ++
   indent (stmt.to_python' p1) ++ ["else:"] ++ indent (stmt.to_python' p2)
| (stmt.while e p1) :=
  ["while " ++ to_string e ++ ":"] ++ indent (stmt.to_python' p1)

def stmt.to_python (s : stmt) :=
concat "\n" (stmt.to_python' s)

def func.to_python' : func → list string
| (func.mk name ps body ret) :=
  ("def " ++ name ++ parens (concat "," ps) ++ ":") :: indent (body.to_python' ++
  ["return " ++ to_string ret])

def func.to_python (f : func) :=
concat "\n" (func.to_python' f)

instance func.has_to_python : has_to_python func :=
⟨ func.to_python ⟩

def prog.to_python' : prog → list string
| (prog.mk fs main) :=
  fs.map func.to_python ++ ["print " ++ main ++ "()"]

def prog.to_python (p : prog) : string :=
  concat "\n\n" (prog.to_python' p)

instance prog.has_to_python : has_to_python prog :=
⟨ prog.to_python ⟩

end imp
