-- Pretty Printing to Lean Syntax

import ..syntax
import ..print_util

namespace sugar

mutual def expr.repr, expr.repr_list
with expr.repr : expr → string
| (expr.val v) := "(expr.val " ++  v.repr ++ ")"
| (expr.var x) := "(expr.var " ++ x ++ ")"
| (expr.op1 op e1) := "(expr.op1 " ++ op.repr ++ e1.repr  ++ ")"
| (expr.op2 op e1 e2) := "(expr.op2 " ++ op.repr ++ " " ++ e1.repr ++ e2.repr ++ ")"
| (expr.length e) := "(expr.length " ++ e.repr ++ ")"
| (expr.index e1 e2) := "(expr.index " ++ e1.repr ++ "[" ++ e2.repr ++ "])"
| (expr.call f args) := "(expr.call " ++ repr f ++ (concat ", " $ expr.repr_list args) ++ ")"
| (expr.alloc x sz) := "(expr.alloc " ++ x.repr ++ sz.repr ++ ")"
with expr.repr_list : list expr → list string
| [] := []
| (e :: es) := expr.repr e :: expr.repr_list es

instance expr.has_repr : has_repr expr :=
⟨ expr.repr ⟩

def stmt.repr_aux : stmt → list string
| (stmt.set x e) := ["(stmt.set " ++ repr x ++ e.repr ++ ")"]
| (stmt.write x e1 e2) := [repr x ++ "[" ++ e1.repr ++ "]" ++ " = " ++ e2.repr ++ ";"]
| (stmt.ifelse e p1 p2) := ["(stmt.ifelse "] ++ stmt.repr_aux p1 ++ stmt.repr_aux p2 ++ [")"]
| (stmt.while g p1) := ["(stmt.while " ++ g.repr] ++ stmt.repr_aux p1 ++ stmt.repr_aux p1 ++ [")"]
| (stmt.seq s1 s2) := stmt.repr_aux s1 ++ stmt.repr_aux s2
| (stmt.ifthen e p1) := ["(stmt.ifthen " ++ e.repr] ++ stmt.repr_aux p1 ++ [")"]
| (stmt.ret e) := ["(stmt.ret " ++ repr e ++ ")"]
| (stmt.expr e) := [repr e]

def stmt.repr (s : stmt) :=
concat "\n" (stmt.repr_aux s)

instance stmt.has_repr : has_repr stmt :=
⟨ stmt.repr ⟩

def func.repr : func → string
| (func.mk name ps body) :=
  parens $ "func.mk " ++ repr name ++ squares  (string.join $ list.intersperse "," $ ps.map repr) ++
   parens (concat "\n" $ indent body.repr_aux)

instance func.has_repr : has_repr func :=
⟨ func.repr ⟩

def prog.repr : prog → string
| (prog.mk funcs main) :=
  concat "\n\n" $ list.map func.repr funcs ++ [main]

instance prog.has_repr : has_repr prog :=
⟨ prog.repr ⟩

end sugar
